# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplateMDSviaR(Template):       
    'Performs classical (metric) multidimensional scaling (MDS) of a data matrix using the <a href="http://stat.ethz.ch/R-manual/R-patched/library/stats/html/00Index.html" target="blank">stats</a> package in R.'

    __version__ = '1.0.0'

    tags = ['MDS' ]
    engines = ['R_script']

    inputs = '''
x = DataMatrix('Random variables: ')
stand = Boolean('Standardize the data? ')
if stand:
    center = Boolean('Substract the means? ')
    scale  = Boolean('Scale each variable? ')
distMethod =  Text('Specify the distance measure: ', ['euclidean', 'squared_euclidean','maximum', 'manhattan', 'canberra', 'binary', 'minkowski'])
if distMethod == 'minkowski':
    p = DataVector('The power of the Minkowski distance: ')

k=Text('The maximum dimension of the space which the data are to be represented in; must be in {1, 2,..., n-1}: ')
eig=Boolean('Return eigenvalues? ')
add=Boolean('An additive constant c* added to the non-diagonal dissimilarities such that all n-1 eigenvalues are non-negative? ')
x_ret=Boolean('Return the doubly centred symmetric distance matrix? ')
'''

    rscript = '''
X = NULL
% for i in range(len(x)):
tt=mydata$${x[i]}
tt[tt==Inf]=NA
X=cbind(X, tt)
% endfor
X=as.matrix(X)
% for i in range(len(x)):
colnames(X)[${i+1}]='${x.name[i]}'
% endfor

% if stand:
% if center:
% if scale:
X=scale(X,center=T,scale=T)
% else:
X=scale(X,center=T,scale=F)
% endif
% else:
% if scale:
#dividing by the root mean square for all r.v.
X=scale(X,center=F,scale=T)
% endif
% endif
% endif


% if distMethod =='squared_euclidean':
dm = dist(X, method='euclidean')^2
% else:
% if distMethod == 'minkowski':
dm= dist(X, method ='${distMethod}', p=${p})
% else:
dm= dist(X, method ='${distMethod}')
% endif
% endif


mds=cmdscale(dm, k=${int(k)},eig=${int(bool(eig))},add=${int(bool(add))},x.ret=${int(bool(x_ret))})

if (is.list(mds)) {
svg('mdsPlot.svg')
plot(mds$points[,1],mds$points[,2],xlab="principal coordinates 1",ylab="principal coordinates 2")
dev.off()

write.dta(data.frame(mds$points),file="mds.dta")
% if eig:
write.dta(data.frame(mds$eig),file="eig.dta")
% endif
% if add:
write.dta(data.frame(mds$ac),file="ac.dta")
% endif
% if x_ret:
write.dta(data.frame(mds$x),file="x_dist.dta")
% endif
} else {
svg('mdsPlot.svg')
plot(mds[,1],mds[,2],xlab="principal coordinates 1",ylab="principal coordinates 2")
dev.off()
write.dta(data.frame(mds),file="mds.dta")
}
'''
