# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplatePCAviaR(Template):       
    'Performs a principal component analysis using the <a href="http://stat.ethz.ch/R-manual/R-patched/library/stats/html/00Index.html" target="blank">stats</a> package in R.'

    __version__ = '1.0.0'

    tags = [ 'PCA' ]
    engines = ['R_script']

    inputs = '''
x = DataMatrix('Random variables: ')
subset=Boolean('Select a subset of data points (in rows)? ')
if bool(subset):
    rowID= DataVector('Subset: ')
    
cor = Boolean('Use correlation matrix? ')
scale = Boolean('Scale each variable? ')

'''

    rscript = '''
<% from EStat.Utils import Rmmult %>\\
%if subset:
PCA=princomp(~${Rmmult(x)},data=mydata,subset=mydata$${rowID},cor=${int(bool(cor))},scale=${int(bool(scale))})
%else:
PCA=princomp(~${Rmmult(x)},data=mydata,cor=${int(bool(cor))},scale=${int(bool(scale))})
%endif

summary(PCA)
myPCA=NULL
dd=dim(PCA$loadings)
myPCA$loadings=as.data.frame(t(matrix(PCA$loadings,dd[1],dd[2])))
%for i in range(len(x)):
colnames(myPCA$loadings)[${i+1}]='${x.name[i]}'
%endfor
myPCA$sd=PCA$sd
myPCA$var=(PCA$sd)^2
myPCA$prop=sapply(1:dd[2],function(x) myPCA$var[x]/sum(myPCA$var))
myPCA$cumprop=sapply(1:dd[2],function(x) sum(myPCA$prop[1:x]))
myPCAsummary=t(rbind(myPCA$sd,myPCA$var,myPCA$prop,myPCA$cumprop))
colnames(myPCAsummary)=c('sd','Variance','prop.','cum.prop.')
myPCA$scores=PCA$scores
 

#A barplot of the eignvalues for PCs
svg('PCAPlots_eigenval.svg')
plot(PCA)
dev.off()

# A biplot of PC_i against PC_j
for (i in 1:${len(x)-1}){
    for (j in (i+1):${len(x)}){
        svgout=paste('PCAPlots_',i,'_',j,'.svg',sep='')
        svg(svgout)
        biplot(PCA,choices =c(i,j))
        dev.off()
    }
}
write.dta(data.frame(myPCA$scores),file="scores.dta") 
write.dta(data.frame(myPCA$loadings),file="loadings.dta")
write.dta(data.frame(myPCAsummary),file="PCAsummary.dta")
'''
